#ifndef ONEDEVICEELECTROPROGONTASK_H
#define ONEDEVICEELECTROPROGONTASK_H

#include <QObject>
#include <QThread>
#include <QTimer>
#include <QEventLoop>
#include <QDateTime>
#include "interface/iwidgetdevice.h"
#include "tools/memloader.h"
#include "tools/channelfinder.h"

///
/// \brief The OneDeviceElectroprogonTask class задача электропрогона для одной платы
///
class OneDeviceElectroprogonTask : public QObject
{
    Q_OBJECT
public:
    ///
    /// \brief OneDeviceElectroprogonTask конструктор
    /// \param revId текущая ревизия платы
    ///
    explicit OneDeviceElectroprogonTask(int devId, int revId, int uid, const ElectroprogonParametres& params, const ChannelFinder::DeviceInfo& devInfo);
    ~OneDeviceElectroprogonTask();
public slots:
    ///
    /// \brief onStart старт задачи
    ///
    void onStart();
    ///
    /// \brief onStop останов задачи
    ///
    void onStop();
private slots:
    ///
    /// \brief onTimer тело корутины учёта времени (шагов) электропрогона
    ///
    void onTimer();
    ///
    /// \brief onTimerElectroprogon тело корутины функционального алгоритма электропрогона
    ///
    void onTimerElectroprogon();
    ///
    /// \brief onTimerOutRk тело корутины функционального алгоритма выдачи рк
    ///
    void onTimerOutRk();
    ///
    /// \brief onTimerInRk лето корутины обработки входных рк
    ///
    void onTimerInRk();
signals:
    ///
    /// \brief signalHasFinished извещение о фактической остановке задачи
    ///
    void signalHasFinished();
    ///
    /// \brief signalSendStatistics отправка наработанной статистики
    /// \param uid идентификатор платы в программе
    /// \param devStat наработанная статистика
    ///
    void signalSendStatistics(int uid, DeviceStatistics devStat);
    ///
    /// \brief signalPrint команда - печать одной строки
    /// \param line строка
    ///
    void signalPrint(QString line);
    ///
    /// \brief signalPrintLines команда - печать группы строк
    /// \param lines группа строк
    ///
    void signalPrintLines(QStringList lines);
private:
    ///
    /// \brief initDevice инициализация девайса
    ///
    void initDevice();
    ///
    /// \brief deinitDevice ДЕинициализация девайса
    ///
    void deinitDevice();
    ///
    /// \brief processingTransmitters обработка передатчиков
    ///
    void processingTransmitters();
    ///
    /// \brief processingReceivers обработка приёмников
    ///
    void processingReceivers();
    ///
    /// \brief writeMem запись MEM файлов в RAM
    /// \return результат операции
    ///
    bool writeMem();
protected:
    const int cTimerPeriod = 1000;              ///< константа - шаг электпрогона 1 сек
    const int cTimerElectroprogonPeriod = 10;   ///< константа - шаг функционального алгоритма 1 мс
    const int cTimerOutRkPeriod = 500;          ///< константа - шаг выдачи рк 0,5 сек
    const int cTimerInRkPeriod = 250;           ///< константа - шаг приёма рк 0,25 сек
    QThread mThread;                            ///< вычислительный поток
    int mCurrentRevId;                          ///< текущая ревизия платы
    int mCurrentDevId;
    QTimer* mTimer { nullptr };                             ///< таймер учёта времени электропрогона
    QTimer* mTimerElectroprogon { nullptr };                ///< таймер-корутина электропрогона
    QTimer* mTimerOutRk { nullptr };                        ///< таймер-корутина выдачи вых рк
    QTimer* mTimerInRk { nullptr };                         ///< таймер-корутина обработки входных рк
    bool mIsStopped;                            ///< признак останова задачи
    bool mIsEndTimer;                           ///< признак останова таймера
    bool mIsEndTimerElectroprogon;              ///< признак останова таймера
    bool mIsEndTimerOutRk;                      ///< признак останова таймера
    int mCurrentUid;                            ///< идентификатор платы в прогамме
    DeviceStatistics mCurrentDevStat;           ///< текущая наработанная статистика
    ElectroprogonParametres mParams;            ///< параметры настроек плат
    ChannelFinder::DeviceInfo mDevInfo;         ///< инфо о плате
    bool mFlagRk;
};

#endif // ONEDEVICEELECTROPROGONTASK_H
