#ifndef IWIDGETDEVICE_H
#define IWIDGETDEVICE_H

#include <QObject>
#include <QVector>
#include <QMap>
#include <cmath>

///
/// \brief The DeviceStatistics struct наработанная статистика платы, набранная за прохождение электропрогона
///
struct DeviceStatistics {
    QVector<int> mTMsg;             ///< сообщения передатчиков
    QVector<int> mTErr;             ///< ошибки передатчиков
    QVector<int> mRMsg;             ///< сообщения приёмников
    QVector<int> mRErr;             ///< ошибки приёмников
    QVector<int> mRkOut;            ///< выходные разовые команды
    QVector<int> mRkIn;             ///< входные разовые команды
};

///
/// \brief The DeviceStatisticsParametres struct параметры наработанной статитсики для платы
///
struct DeviceStatisticsParametres {
    int mCountT;        ///< кол-во передатчиков
    int mCountR;        ///< кол-во приёмников
    int mCountRkIn;
    int mCountRkOut;
    DeviceStatisticsParametres(){}
    ///
    /// \brief DeviceStatisticsParametres конструктор с параметрами
    /// \param countT Кол-во передатчиков
    /// \param countR Кол-во приёмников
    ///
    DeviceStatisticsParametres(int countT, int countR, int countRkIn, int countRkOut):
    mCountT(countT), mCountR(countR), mCountRkIn(countRkIn), mCountRkOut(countRkOut) {}
};

///
/// \brief The ElectroprogonParametres struct параметры настроек плат для электропрогона
///
struct ElectroprogonParametres {
   int mSpeed;      ///< скорость 0 - 100; 1 - 12-14; 2 - 50
   bool mParity;    ///< бит чётности
   bool mReverce;   ///< реверс
   bool mOnRRT;     ///< включение RRT
   int mDiscrRRT;   ///< дискретность RRT 0 - 1; 1 - 10
   int mValueRRT;   ///< значение RRT
   ElectroprogonParametres(){}
   ///
   /// \brief ElectroprogonParametres Конструктор с параметрами
   /// \param speed скорость
   /// \param parity бит паритета
   /// \param reverce нумерация бит лейбла
   /// \param onRRT использование RRT
   /// \param discrRRT дискретность RRT
   /// \param valRRT значение RRT
   ///
   ElectroprogonParametres(int speed, bool parity, bool reverce, bool onRRT, int discrRRT, int valRRT):
   mSpeed(speed), mParity(parity), mReverce(reverce), mOnRRT(onRRT), mDiscrRRT(discrRRT), mValueRRT(valRRT){}
};

#define REV_ID_A429_UD88        0x01        ///< ревизия поддерживаемого устройства UD88
#define REV_ID_A429_UD84        0x11        ///< ревизия поддерживаемого устройства UD84
#define REV_ID_A429_UD44        0x21        ///< ревизия поддерживаемого устройства UD44
#define REV_ID_A429_UD66        0x31        ///< ревизия поддерживаемого устройства UD66
#define REV_ID_A429_UD22        0x41        ///< ревизия поддерживаемого устройства UD22

#define DEVICE_ID_PCIE_A429     0x9477      ///< идентификатор поддерживаемого устройства A429 PCIe
#define DEVICE_ID_MPCIE_A429    0x9478      ///< идентификатор поддерживаемого устройства A429 mPCIe
#define DEVICE_ID_XMC_A429      0x9481      ///< идентификатор поддерживаемого устройства A429 XMC

///
/// \brief The IWidgetDevice class интерфейс виджета, описывающий статистику платы
///
class IWidgetDevice {
public:
    static DeviceStatisticsParametres getDevStatParametres(int devid, int revid) {
        DeviceStatisticsParametres params(0,0,0,0);
        switch (revid) {
            case REV_ID_A429_UD88: params.mCountR = 8; params.mCountT = 8; break;
            case REV_ID_A429_UD84: params.mCountR = 8; params.mCountT = 4; break;
            case REV_ID_A429_UD44: params.mCountR = 4; params.mCountT = 4; break;
            case REV_ID_A429_UD66: params.mCountR = 16; params.mCountT = 16; break;
            case REV_ID_A429_UD22: params.mCountR = 12; params.mCountT = 12; break;
        }
        switch (devid) {
            case DEVICE_ID_PCIE_A429: params.mCountRkIn = 4; params.mCountRkOut = 4; break;
            case DEVICE_ID_MPCIE_A429: params.mCountRkIn = 2; params.mCountRkOut = 2; break;
            case DEVICE_ID_XMC_A429: params.mCountRkIn = 8; params.mCountRkOut = 8; break;
            default:
            {
                using namespace std;
                int cntRk = max(2, min(params.mCountR, params.mCountT) / 2);
                params.mCountRkIn = cntRk;
                params.mCountRkOut = cntRk;
            }
        }
        return params;
    }

    static DeviceStatistics generateDevStat(int devid, int revid) {
        DeviceStatisticsParametres params = IWidgetDevice::getDevStatParametres(devid, revid);
        DeviceStatistics stat;
        for (int i=0; i<params.mCountR; i++) {
            stat.mRMsg.append(0);
            stat.mRErr.append(0);
        }
        for (int i=0; i<params.mCountT; i++) {
            stat.mTMsg.append(0);
            stat.mTErr.append(0);
        }
        for (int i=0; i<params.mCountRkIn; i++) {
            stat.mRkIn.append(0);
        }
        for (int i=0; i<params.mCountRkOut; i++) {
            stat.mRkOut.append(0);
        }
        return stat;
    }

public:
    ///
    /// \brief setDeviceName установить имя девайса
    /// \param name имя девайса
    ///
    virtual void setDeviceName(const QString& name) = 0;
    ///
    /// \brief getDeviceName получить имя девайса
    /// \return имя девайса
    ///
    virtual QString getDeviceName() = 0;
    ///
    /// \brief setDeviceId установить devid
    /// \param devid devid
    ///
    virtual void setDeviceId(int devid) = 0;
    ///
    /// \brief setRevisionId установить revid
    /// \param revid revid
    ///
    virtual void setRevisionId(int revid) = 0;
    ///
    /// \brief setDeviceStatistics установить наработанную статистику платы
    /// \param devStat наработанная статистика
    ///
    virtual void setDeviceStatistics(const DeviceStatistics& devStat) = 0;
    ///
    /// \brief getStringStatistics получить статистику в текстовом виде
    /// \return статистика в текстовом аиде
    ///
    virtual QStringList getStringStatistics() = 0;
protected:
    DeviceStatistics mCurrentStatictics;                                    ///< текущая статистика
};

#endif // IWIDGETDEVICE_H
