#include <QString>
#include <QStringList>
#include <QStringListModel>
#include <QDebug>
#include <QMessageBox>

#include "listnetworkdialog.h"
#include "ui_listnetworkdialog.h"

#include "tools/channelfinder.h"
#include "dialog_extaddresseditor.h"

ListNetworkDialog::ListNetworkDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ListNetworkDialog)
{
    ui->setupUi(this);

    connect(ui->listWidgetDevices, &QListWidget::itemSelectionChanged, this, [=]{
        ui->pushButtonRemove->setEnabled(ui->listWidgetDevices->selectionModel()->hasSelection());
    });

    ui->pushButtonRemove->setEnabled(false);
    refreshDeviceList();

    connect(ui->pushButtonAdd, &QPushButton::clicked, this, &ListNetworkDialog::onAddClicked);
    connect(ui->pushButtonRemove, &QPushButton::clicked, this, &ListNetworkDialog::onRemoveClicked);
    connect(ui->listWidgetDevices, &QListWidget::activated, this, &ListNetworkDialog::onEditClicked);
}

ListNetworkDialog::~ListNetworkDialog()
{
    delete ui;
}

void ListNetworkDialog::refreshDeviceList()
{
    QStringList deviceList = ChannelFinder::getNetworkDevices();
    ui->listWidgetDevices->clear();
    for(const QString &item : deviceList)
        ui->listWidgetDevices->addItem(item);
}

void ListNetworkDialog::onAddClicked()
{
    setEnabled(false);

    Dialog_ExtAddressEditor dialog;
    dialog.setModal(true);
    if(dialog.exec() == QDialog::Accepted)
    {
        auto result = dialog.getResult();
        qDebug() << result.ipAddress << result.port;

        if(!ChannelFinder::addNetworkDevice(result))
            QMessageBox::warning(this, tr("Ошибка соединения"), tr("Не удалось подключиться к\n%1").arg(result), QMessageBox::Close);

        refreshDeviceList();
    }

    setEnabled(true);
}

void ListNetworkDialog::onRemoveClicked()
{
    auto selectedIndexes = ui->listWidgetDevices->selectionModel()->selectedIndexes();
    if(selectedIndexes.empty())
        return;

    QStringList addr;
    for(const QModelIndex &index : selectedIndexes)
        addr << index.data().toString();

    if(QMessageBox::Yes != QMessageBox::question(this, tr("Подтвердите удаление"),
                          tr("Вы уверены, что хотите удалить\n%1").arg(addr.join('\n')),
                          QMessageBox::Yes, QMessageBox::No | QMessageBox::Default))
    {
        return; // отмена операции
    }

    while(!selectedIndexes.empty())
        ChannelFinder::removeNetworkDevice( selectedIndexes.takeLast().row() );

    refreshDeviceList();
}

void ListNetworkDialog::onEditClicked(const QModelIndex &index)
{
    setEnabled(false);

    QStringList list = index.data().toString().split(':');
    QString ipAddr = list.takeFirst();
    int port = list.takeFirst().toInt();

    Dialog_ExtAddressEditor dialog;
    dialog.setModal(true);
    dialog.setDevice({ ipAddr, port });

    if(dialog.exec() == QDialog::Accepted)
    {
        auto result = dialog.getResult();
        qDebug() << result.ipAddress << result.port;

        if(!ChannelFinder::editNetworkDevice(index.row(), result))
              QMessageBox::warning(this, tr("Ошибка соединения"), tr("Не удалось подключиться к\n%1").arg(result), QMessageBox::Close);

        refreshDeviceList();
    }

    setEnabled(true);
}
